/**
 * Copyright 2017 Confluent Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once

#include <stdlib.h>


#ifdef _MSC_VER
/*
 * Windows
 */
#define WIN32_MEAN_AND_LEAN
#include <Windows.h>

#define DLL_EXPORT __declspec(dllexport)

#define IC_UNUSED

int ic_gettimeofday (struct timeval *tv, struct timezone *tz);

#define ic_strdup(str) _strdup(str)
#define ic_strcasecmp(a,b) _stricmp(a, b)




#else
/*
 * POSIX (Linux, OSX, ..) (or gcc, clang, ..)
 */
#include <sys/time.h>

#define DLL_EXPORT

#define IC_UNUSED __attribute__((unused))

#define ic_gettimeofday(TV,TZ) gettimeofday(TV,TZ)

#define ic_strdup(str) strdup(str)
#define ic_strcasecmp(a,b) strcasecmp(a, b)
#endif

#include <stdint.h>
#include <sys/types.h>
#include <stdarg.h>


int ic_vsnprintf (char *str, size_t size, const char *format, va_list ap);

void ic_uuid_str (char *dst, size_t dst_size);


/**
 * @returns the wallclock timestamp as used by the Kafka message timestamps,
 *          the number of milliseconds since 1970-01-01 00:00
 */
static IC_UNUSED __inline int64_t get_wallclock_timestamp (void) {
        struct timeval tv;

        ic_gettimeofday(&tv, NULL);
        return ((int64_t)tv.tv_sec * 1000LLU) + ((int64_t)tv.tv_usec/1000LLU);
}
