/**
 * Copyright 2017 Confluent Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once

#include <stdint.h>

#include "rdsysqueue.h"
#include "platform.h"

#include "monitoring.pb.h"

typedef struct timebucket_s {
        TAILQ_ENTRY(timebucket_s) tb_link;
        TAILQ_ENTRY(timebucket_s) tb_flat_link;

        int64_t tb_base;   /* Time base in milliseconds since epoch. */

        /* Current stats, stored in Protobuf struct as far as possible. */
        monitoring_MonitoringMessage tb_m;
} timebucket_t;

#define TIMEBUCKETS_HISTORY_SIZE   10
#define TIMEBUCKETS_UNKNOWN_WINDOW -1

typedef struct timebuckets_s {
        TAILQ_HEAD(, timebucket_s) *tbs_buckets; /* [tbs_size] hash array.
                                                  * The bucket tailq is sorted
                                                  * to make lookups more
                                                  * efficient. */
        TAILQ_HEAD(, timebucket_s) tbs_flat;     /* flat bucket list */
        int     tbs_size;                    /* tbs_buckets[] array size */
        int64_t tbs_win_size;                /* Window size / sample period */

        int64_t tbs_sequence;                /* Sequence number */

        int64_t  tbs_history[TIMEBUCKETS_HISTORY_SIZE]; /* Window history
                                                         * cyclical array */
        int      tbs_h_i;                    /* Index of next history slot */
        int      tbs_h_sz;                   /* Window history size */

} timebuckets_t;

void timebucket_destroy (timebucket_t *tb);
timebucket_t *timebucket_get (timebuckets_t *tbs, int64_t timestamp);

void timebuckets_init (timebuckets_t *tbs, int size, int64_t win_size);
void timebuckets_clear (timebuckets_t *tbs, int destroy_buckets);
void timebuckets_destroy (timebuckets_t *tbs);

static __inline IC_UNUSED
void timebuckets_history_add (timebuckets_t *tbs, int64_t window) {
        tbs->tbs_history[tbs->tbs_h_i] = window;
        tbs->tbs_h_i = (tbs->tbs_h_i + 1) % tbs->tbs_h_sz;
}
